import argparse
from genericpath import isdir
from multiprocessing import context
import os
import sys
import yaml

from kubernetes import client, config
from kubernetes.client.exceptions import ApiException
from kindplay.kind import start_kind, stop_kind
from kindplay.tools import run_command
from kindplay.helm import helm_deploy_all, helm_prepare


def requirements_check():
    exit_code = 0
    
    kind_status = run_command("kind --version")
    if kind_status["return_code"] != 0:
        print("kind not found, please install kind.")
        exit_code = 1

    helm_status = run_command("helm version")
    if helm_status["return_code"] != 0:
        print("helm not found, please install helm.")
        exit_code = 1

    docker_status = run_command("docker version")
    if docker_status["return_code"] != 0:
        print("docker not found, please install docker.")
        exit_code = 1

    if exit_code:
        sys.exit(exit_code)


def create_namespaces(base_path: str, k8s_core: client.CoreV1Api):
    kubernetes_folder = os.path.join(base_path,"kubernetes")

    for filename in os.listdir(kubernetes_folder):
        if os.path.isdir(os.path.join(kubernetes_folder,filename)):
            try:
                k8s_core.read_namespace(filename)
            except ApiException as error:
                if error.status == 404:
                    namespace_object = client.V1Namespace(
                            metadata=client.V1ObjectMeta(name=filename)
                        )
                    k8s_core.create_namespace(namespace_object)
                    continue
                raise
            except:
                raise


def playground_start(base_path):
    kind_config_file = os.path.join(base_path,"kind.yaml")
    playground_config_file = os.path.join(base_path,"playground.yaml")

    with open(kind_config_file) as fp:
        kind_config = yaml.safe_load(fp)

    with open(playground_config_file) as fp:
        playground_config = yaml.safe_load(fp)

    start_kind(base_path, kind_config, playground_config)

    config.load_kube_config(context=f"kind-{kind_config['name']}")

    k8s_core = client.CoreV1Api()

    create_namespaces(base_path, k8s_core)

    helm_prepare()

    helm_deploy_all(base_path, playground_config["kubernetes"]["priorityCharts"])


def playground_stop(base_path):
    kind_config_file = os.path.join(base_path,"kind.yaml")
    playground_config_file = os.path.join(base_path,"playground.yaml")

    with open(kind_config_file) as fp:
        kind_config = yaml.safe_load(fp)

    with open(playground_config_file) as fp:
        playground_config = yaml.safe_load(fp)

    stop_kind(base_path, kind_config, playground_config)


def main(): 
    requirements_check()

    parser = argparse.ArgumentParser(description='Kind Playground')
    subparsers = parser.add_subparsers(dest='subparser', metavar="command")

    parser_start = subparsers.add_parser('start')
    parser_start.add_argument('path', type=str, help='Playground path')

    parser_stop = subparsers.add_parser('stop')
    parser_stop.add_argument('path', type=str, help='Playground path')

    subparsers.required = True

    args = parser.parse_args()

    base_path = os.path.realpath(args.path)

    if args.subparser == "start":
        playground_start(base_path)

    if args.subparser == "stop":
        playground_stop(base_path)


if __name__ == "__main__":
    main()



import stat
import tempfile
import sys
import os

from kindplay.tools import run_command


def run_scripts(script_list, base_path):
    script_header = "#!/bin/bash\n"
    env = os.environ
    env["PLAYGROUND_PATH"] = base_path

    for script in script_list:
        with tempfile.NamedTemporaryFile() as fp:
            if script.get("description"):
                print(f"- {script.get('description')}")
            fp.file.write(script_header.encode())
            fp.file.write(script["run"].encode())
            fp.file.close()
            script_file = fp.name
            os.chmod(script_file, stat.S_IRWXU)
            run_command(script_file, None, None)


def start_kind(base_path, kind_config, playground_config):
    kind_config_file = os.path.join(base_path,"kind.yaml")

    print("### Preparing local Kind")
    kind_check = f'kind get kubeconfig --name {kind_config["name"]}'
    if run_command(kind_check)["return_code"] != 0:
        kind_start = f"kind create cluster --config {kind_config_file}"
        if run_command(kind_start, None, None)["return_code"] != 0:
            print("Fail to start kind cluster")
            sys.exit(1)
    else:
        print("- Kind cluster already exist, skipping creation")

    print("\n### Running Post Start Scripts")
    run_scripts(playground_config["postStart"], base_path)


def stop_kind(base_path, kind_config, playground_config):
    kind_stop = f"kind delete cluster --name {kind_config['name']}"
    run_command(kind_stop, None, None)

    print("\n### Running Post Stop Scripts")
    run_scripts(playground_config["postStop"], base_path)



import os
import sys

from kindplay.tools import run_command


def helm_prepare():
    '''
        Add one repository to the helm.
    '''
    helm_repo = "helm repo list"
    helm_repo_result = run_command(helm_repo)
    if helm_repo_result["return_code"] != 0:
        helm_add_repo = "helm repo add bitnami https://charts.bitnami.com/bitnami"
        run_command(helm_add_repo, None, None)


def get_workloads(playground_path):
    workloads = dict()
    charts_path = os.path.join(playground_path, "kubernetes")
    for namespace in os.listdir(charts_path):
        for basedir, dirs, files in os.walk(os.path.join(charts_path, namespace)):
            if f"values.yaml" in files:
                release_name = os.path.basename(basedir)
                workloads[f"{namespace}/{release_name}"]= {
                        "release_name": release_name,
                        "namespace": namespace,
                        "chart_dir": basedir,
                        "values_file": os.path.join(basedir, f"values.yaml")
                    }

    return workloads


def helm_deploy(chart_info):
    print( 
            f'\n## Setting up {chart_info["namespace"]}/{chart_info["release_name"]}',
            flush=True
        )

    print("- Helm dependency update", flush=True)
    dep_cmd = f'helm dependency update --skip-refresh {chart_info["chart_dir"]}'
    dep_status = run_command(dep_cmd, stdout=None, stderr=None)
    if dep_status["return_code"] != 0:
        print("Fail to setup helm dependencies.")
        sys.exit(1)

    print("- Helm upgrade", flush=True)
    helm_cmd = f'helm upgrade --install --wait --timeout 10m "{chart_info["release_name"]}" -n "{chart_info["namespace"]}" -f "{chart_info["values_file"]}" {chart_info["chart_dir"]}'
    run_command(helm_cmd, stdout=None, stderr=None)


def helm_deploy_all(playground_path, priority_charts=list()):
    workloads = get_workloads(playground_path)

    for chart_id in priority_charts:
        if workloads.get(chart_id):
            helm_deploy(workloads[chart_id])

    for chart_id in workloads.keys():
        if chart_id not in priority_charts:
            helm_deploy(workloads[chart_id])




import shlex
import os

from subprocess import Popen, PIPE


def run_command(command, stdout=PIPE, stderr=PIPE, cwd=None, env=None):
    """Run command line and return ."""
    command = shlex.split(command)
    cmd_output = ""
    cmd_error = ""
    try:
        cmd = Popen(command, stderr=stderr, stdout=stdout, cwd=cwd, env=env)
        stdout, stderr = cmd.communicate()
        return_code = cmd.returncode
        if stdout:
            cmd_output = stdout.decode()
        if stderr:
            cmd_error = stderr.decode()
    except FileNotFoundError:
        return_code = 1
        cmd_error = f"Command {command[0]} not found"
        if stderr != PIPE:
            print(cmd_error, flush=True)
    except:
        raise

    return {"return_code": return_code, "output": cmd_output, "error": cmd_error}